InstanceExportLookup

Trait InstanceExportLookup 

pub trait InstanceExportLookup { }
Available on crate features component-model and runtime only.
Trait used to lookup the export of a component instance.

This trait is used as an implementation detail of Instance::get_func and related get_* methods. Notable implementors of this trait are:

Note that this is intended to be a wasmtime-sealed trait so it shouldn’t need to be implemented externally.

Dyn Compatibility

This trait is dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementations on Foreign Types

Source

impl InstanceExportLookup for String

Source

impl InstanceExportLookup for str

Source

impl<T> InstanceExportLookup for &T

Implementors