pub trait InstanceExportLookup { }Available on crate features
component-model and runtime only.Expand description
Trait used to lookup the export of a component instance.
This trait is used as an implementation detail of Instance::get_func
and related get_* methods. Notable implementors of this trait are:
strStringComponentExportIndex
Note that this is intended to be a wasmtime-sealed trait so it shouldn’t
need to be implemented externally.
Dyn Compatibility
This trait is dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".